/*
    Copyright (C) 2010 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include<iostream>
#include<cstddef>
#include<xdgbasedirectory.h>
#include "applicationinfo.h"
#include "treewidgetapplication.h"


/******************/
/* Helper methods */

Qt::CheckState getCheckStateFromString(QString &_value) {
    Qt::CheckState checkState = Qt::Unchecked;
    if (_value == "checked")
        checkState = Qt::Checked;
    else if (_value == "partial")
        checkState = Qt::PartiallyChecked;

    return checkState;
}

/*****************/
/* Class methods */

/* Glob */

Glob::Glob(QString &_value, QString &_filter) {
    this->value = new QString(_value);
    if (_filter == "include")
        this->filter = true;
    else
        this->filter = false;
}

Glob::~Glob() {
  //qDebug("\t\t\tGlob: free memory");
  delete this->value;
  this->value = NULL;
}

/* ApplicationDirectory */

ApplicationDirectory::ApplicationDirectory(const QString &_path, const QString &_recursive) {
    this->globs = new QList<Glob *>();
    this->pathList = NULL;
    this->path = NULL;
    if (_recursive == "true") {
        this->recursive = true;
    } else {
        this->recursive = false;
    }

    this->exists = true;

#ifdef __unix__
    wordexp_t wep;
    char **output = NULL;
    int ret = 0;
    if (!_path.isEmpty()) {
        this->path = new QString(_path);
	this->path->replace(" ", "?");
        this->pathList = new QList<QDir*>();

        ret = wordexp(this->path->toStdString().c_str(), &wep, WRDE_SHOWERR | WRDE_UNDEF); //  | WRDE_APPEND);
        output = wep.we_wordv;
        for (size_t i=0; ret == 0 && i < wep.we_wordc; i++) {
	  QDir *dir = new QDir(output[i]);
	  if (dir->exists()) {
	    pathList->append(dir);
	  } else {
	    delete dir;
	  }
        }
        wordfree(&wep);
        output = NULL;

	if (pathList->empty()) {	
	  delete this->pathList;
	  this->pathList = NULL;
	}
    }

    if (NULL == this->pathList) {
      this->exists = false;
    }

#endif
}

ApplicationDirectory::~ApplicationDirectory() {
//    std::cout << "\t\tApplicationDirectory: free memory" << std::endl;

    while (!this->globs->isEmpty()) {
        delete this->globs->takeFirst();
    }
    delete this->globs;
    this->globs = NULL;
    if (NULL != this->path) {
        delete this->path;
        this->path = NULL;
    }
    if (NULL != this->pathList) {
      while (!this->pathList->isEmpty()) {
	delete this->pathList->takeFirst();
      }
      delete this->pathList;
      this->pathList = NULL;
    }
}

qint64 ApplicationDirectory::addFileToList(QTreeWidget *treeWidget) const {
  qint64 filesSize = 0;
  if (NULL != treeWidget && this->exists) {
    
    // Add include files
#ifdef __unix__
    wordexp_t wep;
    char **output = NULL;
    int ret = 0;
    
    for (int i=0; i < pathList->size(); i++) {
      for (int j=0; j < globs->size(); j++) {
	if (globs->at(j)->isInclusive()) {
	  QString strGlobPath = pathList->at(i)->absolutePath() + QDir::separator();
	  strGlobPath += globs->at(j)->getValue(); // compilation error if on one line
	  strGlobPath.replace("//", "/");
	  strGlobPath.replace(" ", "?"); // wordexp does not seems to accept space in a path.

	  // qDebug("addFileToList: %s", strGlobPath.toStdString().c_str());

	  ret = wordexp(strGlobPath.toStdString().c_str(), &wep, WRDE_SHOWERR | WRDE_UNDEF);
	  output = wep.we_wordv;

	  // if explanation:
	  // if "$HOME/.wine/drive_c/windows/temp/" does not contains files, wordexp return the strGlobPath string
	  // like: "/home/yoyodine/.wine/drive_c/windows/temp/*". But it is not a valid path.
	  if (ret == 0 && (wep.we_wordc > 1 || (wep.we_wordc == 1 && strGlobPath != output[0]))) {
	    for (size_t k=0; k < wep.we_wordc; k++) {
	      // qDebug("output[k]=%s", output[k]);
	      QFileInfo *fileInfo = new QFileInfo(QString(output[k]));
	      if (fileInfo->isDir() && recursive) {
		QDir qDir(fileInfo->absoluteFilePath());
		// qDebug("addDirectoryToList 1st call: %s", fileInfo->absoluteFilePath().toStdString().c_str());
		filesSize += this->addDirectoryToList(treeWidget, qDir);
	      }
	      TreeWidgetFile *twFile = new TreeWidgetFile(treeWidget, fileInfo);
	      filesSize += twFile->getFileSize();
	    }
	  }
	  wordfree(&wep);
	  output = NULL;
	}
      }
      
    }
    
#endif
    // Remove exclude files
  }
  return filesSize;
}


qint64 ApplicationDirectory::addDirectoryToList(QTreeWidget *treeWidget, const QDir &qDir) const {
  qint64 filesSize = 0;
  QFileInfoList listFileInfo = qDir.entryInfoList();

  //qDebug("addDirectoryToList: %s", qDir.absolutePath().toStdString().c_str());
  
  QFileInfoList::const_iterator ite = listFileInfo.constBegin();
  while (ite != listFileInfo.constEnd()) {
    QFileInfo fileInfo = *ite;
    QString fileName = fileInfo.fileName();
    if (fileName != "."  && fileName != "..") { // exclude parent and current path from the list
      // qDebug("addDirectoryToList: %s", fileInfo.absoluteFilePath().toStdString().c_str());
      if (ite->isDir()) {
	filesSize += this->addDirectoryToList(treeWidget, QDir(fileInfo.absoluteFilePath()));
      }
      TreeWidgetFile *twFile = new TreeWidgetFile(treeWidget, fileInfo.absoluteFilePath());
      filesSize += twFile->getFileSize();
    }
    ++ite;
  }

  return filesSize;
}

/* ApplicationCategory */

ApplicationCategory::ApplicationCategory() {

    this->name = NULL;
    this->icon = NULL;
    this->directories = NULL;
    checked = Qt::Unchecked;
}

ApplicationCategory::~ApplicationCategory() {
//    std::cout << "\tApplicationCategory: free memory" << std::endl;

    if (NULL != this->name) {
        delete this->name;
        this->name = NULL;
    }

    if (NULL != this->icon) {
        delete this->icon;
        this->icon = NULL;
    }

    if (NULL != this->directories) {
        while (!this->directories->isEmpty()) {
            delete this->directories->takeFirst();
        }
        delete this->directories;
        this->directories = NULL;
    }
}

void ApplicationCategory::addDirectory(ApplicationDirectory *directory) {
    if (NULL == directories) {
        directories = new QList<ApplicationDirectory *>();
    }

    if (NULL != directory) {
        directories->append(directory);
    }
}

bool ApplicationCategory::isDisplayable() const {
  bool ret = false;
  for (int i=0; !ret && i < this->directories->size(); i++) {
    ret = this->directories->at(i)->isAvailable();
  }
 
  return ret;
}

/* ApplicationInfo */

ApplicationInfo::ApplicationInfo(const QFileInfo *inputFile) {

    this->displayable = true;
    this->name = NULL;
    this->icon = NULL;
    this->categories = NULL;
    checked = Qt::Unchecked;

    QFile file(inputFile->absoluteFilePath());
    this->displayable = file.open(QFile::ReadOnly);
    if (this->displayable) {
        this->displayable = file.isReadable();
    }

    if (this->displayable) {
        this->displayable = this->readFile(&file);
    }

    if (this->displayable) {
        bool haveCatToDisplay = false;
        for (int i=0; !haveCatToDisplay && i<this->categories->size(); i++) {
            haveCatToDisplay = this->categories->at(i)->isDisplayable();
        }
        this->displayable = haveCatToDisplay;
    }

    if (!this->displayable) {
        freeCategories();
    }

    file.close();
}

ApplicationInfo::~ApplicationInfo() {

//    std::cout << "ApplicationInfo: free memory" << std::endl;

    // free memory
    freeCategories();

    if (NULL != this->name) {
        delete this->name;
        this->name = NULL;
    }
    if (NULL != this->icon) {
        delete this->icon;
        this->icon = NULL;
    }
}

void ApplicationInfo::freeCategories() {
    if (NULL != this->categories) {
        while (!this->categories->isEmpty()) {
            delete this->categories->takeFirst();
        }
        delete this->categories;
        this->categories = NULL;
    }
}

void ApplicationInfo::setName(const QString &_name) {
    if (NULL != this->name) {
        qWarning("Application name already define!");
        delete this->name;
    }
    this->name = new QString(_name);
}

void ApplicationInfo::setIcon(const QString &_icon) {
    if (NULL != this->icon) {
        qWarning("Application icon already define!");
        delete this->icon;
    }
    this->icon = new QString(_icon);
}

void ApplicationInfo::addCategory(ApplicationCategory *category) {
    if (NULL == this->categories) {
        this->categories = new ListCategories();
    }
    if (NULL != category) {
        this->categories->append(category);
    }
}

bool ApplicationInfo::readFile(QFile *file) {
    QXmlSimpleReader xmlReader;
    QXmlInputSource *source = new QXmlInputSource(file);
    if (NULL == source) {
        return false;
    }

    xmlReader.setContentHandler(this);
    xmlReader.setErrorHandler(this);

    bool readOK = xmlReader.parse(source);
    if (readOK) {

    } else {
        qErrnoWarning("parsing failed");
    }

    delete source;

    return readOK;
}


ApplicationCategory *category = NULL;
ApplicationDirectory *directory = NULL;
bool ApplicationInfo::startElement (const QString __attribute__((unused)) &namespaceURI, const QString &localName, const QString __attribute__((unused)) &qName, const QXmlAttributes &attributes) {

    bool ret = true;
//    std::cout << "START localName=" << localName.toStdString() << std::endl;
    if (localName == "application") {
        QString name = attributes.value("name");
        QString selection = attributes.value("selection");
        QString icon = attributes.value("icon");
        if (name.isEmpty()) {
            ret = false;
        } else {
            this->setName(name);
            this->setChecked(getCheckStateFromString(selection));
            this->setIcon(icon);
        }
    } else if (localName == "category") {
        QString name = attributes.value("name");
        QString selection = attributes.value("selection");
        QString icon = attributes.value("icon");
        if (name.isEmpty()) {
            ret = false;
        } else {
            category = new ApplicationCategory();
            this->addCategory(category);

            category->setName(name);
            category->setChecked(getCheckStateFromString(selection));
            category->setIcon(icon);
        }
    } else if (localName == "directory") {
        QString name = attributes.value("name");
        QString recursive = attributes.value("recursive");
        if (name.isEmpty() || NULL == category) {
            ret = false;
        } else {
            directory = new ApplicationDirectory(name, recursive);
            category->addDirectory(directory);
        }
    } else if (localName == "glob") {
        QString filter = attributes.value("filter");
        QString value = attributes.value("value");
        if (name->isEmpty() || filter.isEmpty() || NULL == directory) {
            ret = false;
        } else {
            directory->globs->append(new Glob(value, filter));
        }
    }

    return ret;
}

bool ApplicationInfo::endElement (const QString __attribute__((unused)) &namespaceURI, const QString &localName, const QString __attribute__((unused)) &qName) {

    static const char errorMsg[] = "Error in XML, memleak could happened";
//    std::cout << "END localName=" << localName.toStdString() << std::endl;
    bool ret = true;

    if (localName == "application") {
        if (directory != NULL || category != NULL) {
            std::cerr << errorMsg << std::endl;
            ret = false;
        }
    } else if (localName == "category") {
        if (NULL == category) {
            std::cerr << errorMsg << std::endl;
            ret = false;
        }
        category = NULL;
    } else if (localName == "directory") {
        if (NULL == directory) {
            std::cerr << errorMsg << std::endl;
            ret = false;
        }
        directory = NULL;
    } else if (localName == "glob") {
        if (directory == NULL || category == NULL) {
            std::cerr << errorMsg << std::endl;
            ret = false;
        }
    }

    return ret;
}

bool ApplicationInfo::fatalError (const QXmlParseException &exception) {

    //std::cout << "fatal error: " << exception.message().toStdString() << std::endl;
    qWarning("%s", exception.message().toStdString().c_str());
    return false;
}
