/*
  Copyright (C) 2010 Yvon TANGUY

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<stdlib.h>
#include<typeinfo>
#include<iostream>
#include<QFile>
#include<QDir>
#include<QMessageBox>
#include<QIcon>
#include<QSplitter>
#include<QDesktopWidget>
#include<icontheme.h>
#include<iconnaming.h>
#include "mainwindow.h"
#include "sccutils.h"
#include "treewidgetapplication.h"

MainWindow::MainWindow() : QMainWindow() {
  this->setupUi();
  this->retranslateUi();
  this->setupEventsUi();
  this->setupIconsUi();

  this->initAppList();
}

MainWindow::~MainWindow() {

}

void MainWindow::setupUi() {
  if (this->objectName().isEmpty()) {
    this->setObjectName(QString::fromUtf8("MainWindow"));
  }
  this->resize(640, 480);
  // Move this app in the screen center:
  QDesktopWidget *desktop = QApplication::desktop();
  this->move((desktop->width() - this->width())/2, (desktop->height() - this->height())/2);

  centralWidget = new QWidget(this);
  centralWidget->setObjectName(QString::fromUtf8("centralWidget"));

  treeWidgetCheckApp = new QTreeWidget(centralWidget);
  treeWidgetCheckApp->setObjectName(QString::fromUtf8("treeWidgetCheckApp"));
  treeWidgetCheckApp->setColumnCount(1);

  treeWidgetListFiles = new QTreeWidget(centralWidget);
  treeWidgetListFiles->setColumnCount(2);
  treeWidgetListFiles->setObjectName(QString::fromUtf8("treeWidgetListFiles"));

  progressBar = new QProgressBar(centralWidget);
  progressBar->setObjectName(QString::fromUtf8("progressBar"));
  progressBar->setValue(0);
  progressBar->setVisible(false);

  QSplitter *splitter = new QSplitter(centralWidget);
  splitter->addWidget(treeWidgetCheckApp);
  splitter->addWidget(treeWidgetListFiles);

  QVBoxLayout *layoutCentralWidget = new QVBoxLayout(centralWidget);
  layoutCentralWidget->setSizeConstraint(QLayout::SetMaximumSize);
  layoutCentralWidget->addWidget(splitter);
  layoutCentralWidget->addWidget(progressBar);
  layoutCentralWidget->setSpacing(0);
  layoutCentralWidget->setContentsMargins(0, 0, 0, 0);

  toolBar = new QToolBar(this);
  toolBar->setObjectName(QString::fromUtf8("toolBar"));
  statusBar = new QStatusBar(this);
  statusBar->setObjectName(QString::fromUtf8("statusBar"));

  this->setCentralWidget(centralWidget);
  this->addToolBar(Qt::TopToolBarArea, toolBar);
  this->setStatusBar(statusBar);

  actionAnalyze = new QAction(this);
  actionSelectAll = new QAction(this);
  actionClean = new QAction(this);
  actionQuit = new QAction(this);
  actionAbout = new QAction(this);

  actionAnalyze->setObjectName(QString::fromUtf8("actionAnalyze"));
  actionSelectAll->setObjectName(QString::fromUtf8("actionSelectAll"));
  actionClean->setObjectName(QString::fromUtf8("actionClean"));
  actionQuit->setObjectName(QString::fromUtf8("actionQuit"));
  actionAbout->setObjectName(QString::fromUtf8("actionAbout"));

  toolBar->addAction(actionAnalyze);
  toolBar->addAction(actionSelectAll);
  toolBar->addAction(actionClean);
  toolBar->addSeparator();
  toolBar->addAction(actionAbout);
  toolBar->addAction(actionQuit);

}

void MainWindow::retranslateUi() {
  this->setWindowTitle(QApplication::translate("MainWindow", "SimpleFileCleaner", 0, QApplication::UnicodeUTF8));

  actionAnalyze->setText(QApplication::translate("MainWindow", "Analyze (Ctrl+R)", 0, QApplication::UnicodeUTF8));
  actionAnalyze->setShortcut(QApplication::translate("MainWindow", "Ctrl+R", 0, QApplication::UnicodeUTF8));

  actionSelectAll->setText(QApplication::translate("MainWindow", "Select all (Ctrl+A)", 0, QApplication::UnicodeUTF8));
  actionSelectAll->setShortcut(QApplication::translate("MainWindow", "Ctrl+A", 0, QApplication::UnicodeUTF8));

  actionClean->setText(QApplication::translate("MainWindow", "Clean (Ctrl+D)", 0, QApplication::UnicodeUTF8));
  actionClean->setShortcut(QApplication::translate("MainWindow", "Ctrl+D", 0, QApplication::UnicodeUTF8));

  actionQuit->setText(QApplication::translate("MainWindow", "Quit (Ctrl+Q)", 0, QApplication::UnicodeUTF8));
  actionQuit->setShortcut(QApplication::translate("MainWindow", "Ctrl+Q", 0, QApplication::UnicodeUTF8));

  actionAbout->setText(QApplication::translate("MainWindow", "About", 0, QApplication::UnicodeUTF8));
  actionAbout->setToolTip(QApplication::translate("MainWindow", "About box", 0, QApplication::UnicodeUTF8));

  QTreeWidgetItem *treeHeader = treeWidgetCheckApp->headerItem();
  treeHeader->setText(0, QApplication::translate("MainWindow", "Program", 0, QApplication::UnicodeUTF8));
  treeHeader = treeWidgetListFiles->headerItem();
  treeHeader->setText(0, QApplication::translate("MainWindow", "Filename", 0, QApplication::UnicodeUTF8));
  treeHeader->setText(1, QApplication::translate("MainWindow", "Size", 0, QApplication::UnicodeUTF8));
}

void MainWindow::setupEventsUi() {
  QMetaObject::connectSlotsByName(this);
  QObject::connect(treeWidgetCheckApp, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(treeAppChange(QTreeWidgetItem*,int)));
  
}

void MainWindow::setupIconsUi() {
  if (NULL == freedesktop::IconTheme::getCurrentTheme()) {
    // try fetch icons with QT
    this->actionAnalyze->setIcon(QIcon::fromTheme(QString(freedesktop::IconNamingActions::VIEW_REFRESH.c_str())));
    this->actionSelectAll->setIcon(QIcon::fromTheme(QString(freedesktop::IconNamingActions::EDIT_SELECT_ALL.c_str())));
    this->actionAbout->setIcon(QIcon::fromTheme(QString(freedesktop::IconNamingActions::HELP_ABOUT.c_str())));
    this->actionAnalyze->setIcon(QIcon::fromTheme(QString(freedesktop::IconNamingActions::EDIT_CLEAR.c_str())));
    this->actionQuit->setIcon(QIcon::fromTheme(QString(freedesktop::IconNamingActions::APPLICATION_EXIT.c_str())));
  } else {
    freedesktop::IconTheme::setIconsFilesFormats(ICON_FILE_FORMAT_PNG | ICON_FILE_FORMAT_SVG | ICON_FILE_FORMAT_XPM);
    this->actionAnalyze->setIcon(SCCUtils::getIcon(freedesktop::IconNamingActions::VIEW_REFRESH, 32));
    this->actionSelectAll->setIcon(SCCUtils::getIcon(freedesktop::IconNamingActions::EDIT_SELECT_ALL, 32));
    this->actionQuit->setIcon(SCCUtils::getIcon(freedesktop::IconNamingActions::APPLICATION_EXIT, 32));
    this->actionClean->setIcon(SCCUtils::getIcon(freedesktop::IconNamingActions::EDIT_CLEAR, 32));
    this->actionAbout->setIcon(SCCUtils::getIcon(freedesktop::IconNamingActions::HELP_ABOUT, 32));
  }
}

/* events */

/**
 * Permit to select parent when a child is selected, in the app tree view
 */
void MainWindow::treeAppChange(QTreeWidgetItem* item,int __attribute__((unused)) col) {
  static bool updateInProgress = false;

  if (updateInProgress) {
    // Do nothing if an update is already in progress to avoid an infinite loop
    return;
  }

  // Update check states:
  updateInProgress = true;
  if (typeid(* item) == typeid(TreeWidgetApplication)) {
    TreeWidgetApplication *twA = static_cast<TreeWidgetApplication*>(item);
    twA->updateChildCheckState();
  } else if (typeid(* item) == typeid(TreeWidgetCategory)) {
    TreeWidgetCategory *twC = static_cast<TreeWidgetCategory*> (item);
    TreeWidgetApplication *twA = static_cast<TreeWidgetApplication*>(twC->parent());
    twA->updateCheckState();
  }

  updateInProgress = false;
}

void MainWindow::changeEvent(QEvent *e) {
  QMainWindow::changeEvent(e);
  switch (e->type()) {
  case QEvent::LanguageChange:
    this->retranslateUi();
    break;
  default:
    break;
  }
}

/**
 * To make short... what I can say ? Kick this application ass out of memory ?
 */
void MainWindow::on_actionQuit_triggered() {
  QApplication::quit();
}

/**
 * Select all applications and sub applications parts to the list of clean directories searchs
 */
void MainWindow::on_actionSelectAll_triggered() {
  static Qt::CheckState checkState = Qt::Unchecked;
  if (Qt::Unchecked == checkState)
    checkState = Qt::Checked;
  else
    checkState = Qt::Unchecked;

  for (int i=0; i<this->treeWidgetCheckApp->topLevelItemCount(); i++) {
    TreeWidgetApplication *twApp = (TreeWidgetApplication *) this->treeWidgetCheckApp->topLevelItem(i);
    // Note: sub categories will be automatically checked with the event.
    twApp->setCheckState(0, checkState);
  }
}

/**
 * Search for files to delete, and update the file list (treeWidgetListFiles)
 */
void MainWindow::on_actionAnalyze_triggered() {
  int max = 0;
  qint64 filesSize = 0;
  this->actionAbout->setEnabled(false);
  this->actionAnalyze->setEnabled(false);
  this->actionClean->setEnabled(false);
  this->actionQuit->setEnabled(false);
  this->treeWidgetListFiles->setUpdatesEnabled(false);
  this->treeWidgetListFiles->clear();

  for (int i=0; i<this->treeWidgetCheckApp->topLevelItemCount(); i++) {
    TreeWidgetApplication *twApp = (TreeWidgetApplication *) this->treeWidgetCheckApp->topLevelItem(i);
    max += twApp->getDirectoriesCount();
  }
  this->progressBar->setRange(0, max);
  this->progressBar->setValue(0);
  this->progressBar->setVisible(true);


  for (int i=0; i<this->treeWidgetCheckApp->topLevelItemCount(); i++) {
    TreeWidgetApplication *twApp = (TreeWidgetApplication *) this->treeWidgetCheckApp->topLevelItem(i);
    filesSize += twApp->initFilesWidgets(this->treeWidgetListFiles, this->progressBar);
  }

  this->progressBar->setVisible(false);
  this->progressBar->setValue(0);
  this->treeWidgetListFiles->setUpdatesEnabled(true);
  this->actionAbout->setEnabled(true);
  this->actionAnalyze->setEnabled(true);
  this->actionClean->setEnabled(true);
  this->actionQuit->setEnabled(true);

  QString message = QString(tr("Total size: %1 - Number of items: %2")).
    arg(SCCUtils::getHumanSize(filesSize)).
    arg(this->treeWidgetListFiles->topLevelItemCount());
  this->statusBar->showMessage(message);
}

/**
 * Delete files founds in the files list.
 */
void MainWindow::on_actionClean_triggered() {

  int itemCount = this->treeWidgetListFiles->topLevelItemCount();

  // Do nothing if the list is empty
  if (itemCount == 0) {
    this->statusBar->showMessage(tr("Nothing to clean"));
    return;
  }

  this->statusBar->showMessage(tr("Cleaning..."));

  qWarning("SimpleCacheCleaner: Cleaning...");
  int failCount = 0; // For the summarize in the status bar

  int updateInterval = 1;
  if (itemCount > 20) { // SIGFPE, Arithmetic exception if itemCount < 20
    // update the progress bar every 5%
    updateInterval = itemCount/20;
  }

  // Delete files and add display files to the listItemsToDelete list.
  QList<TreeWidgetFile *> *listItemsToDelete = new QList<TreeWidgetFile *>();
  
  // Reset the progress bar
  this->progressBar->setRange(0, itemCount+1);
  this->progressBar->setValue(0);
  this->progressBar->setFormat("%p%");
  this->progressBar->setVisible(true);

  // Delete files loop 
  for (int i=0; i<itemCount; i++) {
    TreeWidgetFile *twF = (TreeWidgetFile *) this->treeWidgetListFiles->topLevelItem(i);
    bool isDeleted = twF->deleteFile();
    if (isDeleted) {
      listItemsToDelete->append(twF);
    } else {
      failCount++;
    }

    // Do not do a too frequent update of the progress bar, it slows too much the process
    if ((i % updateInterval) == 0) {
      this->progressBar->setUpdatesEnabled(false);
      this->progressBar->setFormat(twF->getFileName() + " - %p%");
      this->progressBar->setValue(i);
      this->progressBar->setUpdatesEnabled(true);
      this->progressBar->repaint(0, 0, this->progressBar->width(), this->progressBar->height());
    }
  }

  // Delete files widgets
  this->progressBar->setValue(itemCount);
  this->progressBar->setFormat("Almost done - %p%");
  this->treeWidgetListFiles->setUpdatesEnabled(false);
  while(!listItemsToDelete->isEmpty()) {
    TreeWidgetFile *twF = listItemsToDelete->takeFirst();
    delete twF;
  }

  // Hide the progress bar
  this->progressBar->setVisible(false);
  // Redraw the tree list files
  this->treeWidgetListFiles->setUpdatesEnabled(true);
  // Delete the list
  delete listItemsToDelete;
  // Update the status bar
  this->statusBar->showMessage(tr("Deleted %1 - Failed %2").arg(itemCount-failCount).arg(failCount));
}

void MainWindow::initAppList() {
  static const QString shareAppDir = "share/simple-cache-cleaner/applications";

  QDir appDir(shareAppDir);
  if (!appDir.exists()) {
    QString errorMessage = tr("Share dir not found: %1").arg(shareAppDir);
    QMessageBox::warning(0, "SimpleCacheCleaner",  errorMessage, QMessageBox::Ok);
    qFatal("%s", errorMessage.toStdString().c_str());
  }

  // Read the applications .xml files
  appDir.setFilter(QDir::Files | QDir::NoSymLinks);
  QFileInfoList fileList = appDir.entryInfoList();
  ApplicationInfo *appInfo = NULL;
  for (int i=0; i < fileList.size(); i++) {
    QFileInfo fileInfo = fileList.at(i);
    qDebug("SimpleCacheCleaner: Loading %s", fileInfo.fileName().toStdString().c_str());
    appInfo = new ApplicationInfo(&fileInfo);
    if (appInfo && appInfo->isDisplayable()) {
      new TreeWidgetApplication(this->treeWidgetCheckApp, appInfo);
      appInfo = NULL;
    } else {
      if (NULL != appInfo) {
	delete appInfo;
	appInfo = NULL;
      }
      QString errorMessage = tr("You do not need %1").arg(fileInfo.fileName());
      qWarning("%s", errorMessage.toStdString().c_str());
    }

  }
}
