/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<QObject>
#include<QDebug>
#include<string>
#include "sccutils.h"
#include "fdexception.h"
#include "icontheme.h"

QMap<QString, QString> SCCUtils::mapCache;

SCCUtils::SCCUtils() {
}

QString SCCUtils::getHumanSize(const qint64 bytes) {
    const qint64 kb = 1024;
    const qint64 mb = 1024 * kb;
    const qint64 gb = 1024 * mb;
    const qint64 tb = 1024 * gb;

    if (bytes >= tb)
        return QObject::tr("%1 TB").arg(QLocale().toString(qreal(bytes) / tb, 'f', 3));
    else if (bytes >= gb)
        return QObject::tr("%1 GB").arg(QLocale().toString(qreal(bytes) / gb, 'f', 2));
    else if (bytes >= mb)
        return QObject::tr("%1 MB").arg(QLocale().toString(qreal(bytes) / mb, 'f', 1));
    else if (bytes >= kb)
        return QObject::tr("%1 KB").arg(QLocale().toString(bytes / kb));
    else
        return QObject::tr("%1 bytes").arg(QLocale().toString(bytes));
}

/* Icons helpers */

const QPixmap &SCCUtils::getIcon(const QString &iconFilename) {
    static QPixmap icon;
    icon.load(iconFilename);
    return icon;
}

const QPixmap &SCCUtils::getIcon(const QString &iconName, int size) {
    static const QPixmap icon;
    if (!iconName.isEmpty()) {
        return getIcon(getFilename(iconName, size));
    }
    return icon;
}

const QPixmap &SCCUtils::getIcon(const std::string &iconName, int size) {
    static const QPixmap icon;
    if (!iconName.empty()) {
        return getIcon(getFilename(iconName, size));
    }

    return icon;
}

// FIXME: Not a good code folowing (too slow):
QString &SCCUtils::getFilename(const std::string &iconName, int size) {
    QString key = QString::fromStdString(iconName) + QString::number(size);
    if (!SCCUtils::mapCache.contains(key)) {
        try {
            SCCUtils::mapCache[key] = QString::fromStdString(freedesktop::IconTheme::findIcon(iconName, size));
        } catch (freedesktop::FDException fde) {
            qWarning("%s", fde.what());
            fde.addTrace(__FILE__, __LINE__, __func__);
            fde.printBackTrace();
            SCCUtils::mapCache[key] = "";
        }
    }
    return SCCUtils::mapCache[key];
}

QString &SCCUtils::getFilename(const QString &iconName, int size) {
    QString key = iconName + QString::number(size);
    if (!SCCUtils::mapCache.contains(key)) {
        try {
            SCCUtils::mapCache[key] = QString::fromStdString(freedesktop::IconTheme::findIcon(iconName.toStdString(), size));
        } catch (freedesktop::FDException fde) {
            qWarning("%s", fde.what());
            fde.addTrace(__FILE__, __LINE__, __func__);
            fde.printBackTrace();
            SCCUtils::mapCache[key] = "";
        }
    }
    return SCCUtils::mapCache[key];
}

/*
QFileInfoList *SCCUtils::fileExp(QFileInfoList &infoList, const QString &path) {
  if (!path.isEmpty()) {
    QString tmpPath;
    
    for (int i=0; i<path.size(); i++) {
      switch (path.at(i)) {
      case '\\':
	qWarning("Backslash in path not supported: %s", path);
	break;
      case '$':
	break;
      }
    }
  }

  return &infoList;
}
*/
