/*
    Copyright (C) 2010,2011 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef TREEWIDGETAPPLICATION_H
#define TREEWIDGETAPPLICATION_H

#include<QTreeWidget>
#include<QTreeWidgetItem>
#include<QProgressBar>
#include<QFileInfo>
#include "applicationinfo.h"

#define NAME_TREEWIDGETAPPLICATION "TreeWidgetApplication"
#define NAME_TREEWIDGETCATEGORY "TreeWidgetCategory"

class TreeWidgetApplication : public QTreeWidgetItem {
public:
    TreeWidgetApplication(QTreeWidget *parent, ApplicationInfo *applicationInfo);
    ~TreeWidgetApplication();

    /**
      Init the file list found to delete
      \param fileTW Thee tree widget for display the file list
      \param progressBar The progress bar widget to update
      \return The size of all files found
      */
    qint64 initFilesWidgets(QTreeWidget *fileTW, QProgressBar *progressBar) const;

    /**
      \return The number of directories selected
      */
    int getDirectoriesCount() const;

    void updateCheckState();
    void updateChildCheckState();

private:
    ApplicationInfo *applicationInfo;
};

class TreeWidgetCategory : public QTreeWidgetItem {
public:
    TreeWidgetCategory(QTreeWidgetItem *parent, ApplicationCategory *category);
    ~TreeWidgetCategory();

    /**
      Init the file list found to delete
      \param fileTW Thee tree widget for display the file list
      \param progressBar The progress bar widget to update
      \return The size of all files found
      */
    qint64 initFilesWidgets(QTreeWidget *fileTW, QProgressBar *progressBar) const;

    /**
      \return The number of directories selected
      */
    int getDirectoriesCount() const;

private:
    ApplicationCategory *category;
};

class TreeWidgetFile : public QTreeWidgetItem {
public:
    TreeWidgetFile(QTreeWidget *parent, QString filename);
    TreeWidgetFile(QTreeWidget *parent, QFileInfo *file);
    ~TreeWidgetFile();

    qint64 getFileSize() const;

    bool deleteFile();
    QString getFileName() const;
private:
    void init();
    QFileInfo *fileInfo;
};

#endif // TREEWIDGETAPPLICATION_H
