/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<iostream>
#include<fstream>
#include<algorithm>
#include "inistylefile.h"

using namespace std;
using namespace freedesktop;

#ifdef __unix__
#elif WIN32
#endif

IniStyleFile::IniStyleFile(string filename) {
    this->fileName = filename;
    this->sections = NULL;
//#ifdef DEBUG_MEMORY
//    cout << "IniStyleFile(" << this->fileName << ")" << endl;
//#endif
}

IniStyleFile::~IniStyleFile() {
//#ifdef DEBUG_MEMORY
//    cout << "~IniStyleFile(" << this->fileName << ")" << endl;
//#endif
    clearSections();
    this->fileName.clear();
}

void IniStyleFile::clearSections() {

    if (this->sections != NULL) {
//#ifdef DEBUG_MEMORY
//        cout << "\tfree ini sections " << this->fileName << endl;
//#endif
        while (!this->sections->empty()) {
            IniSections::iterator itClear= this->sections->begin();
//#ifdef DEBUG_MEMORY
//                cout << "\t\tdel section " << (*itClear).first << endl;
//#endif
            delete (*itClear).second;
            this->sections->erase(itClear);
        }
        delete this->sections;
        this->sections = NULL;
    }
}

void IniStyleFile::readFile() throw(FDException) {

    clearSections();

    ifstream fileStream(fileName.c_str(), ios::in);

    if (fileStream.is_open()) {
        this->sections = new IniSections();

        string line;
        IniKeysValues *currentSection = NULL;
        while (fileStream.good()) {
            getline(fileStream, line);

            if (line.empty() || line.at(0) == '#' || line.at(0) == ';') {
                // ignore empty lines and comments
                continue;
            }

            string::size_type posEquals = line.find('=', 0);
            if (line.at(0) == '[' && line.at(line.length()-1) == ']') {
                // Start a section
                string sectionName = line.substr(1, line.length()-2);
                currentSection = new IniKeysValues();
                sections->insert(pair<string, IniKeysValues *>(sectionName, currentSection));
//                (*sections)[sectionName] = currentSection; // what the best ?
            } else if (posEquals != string::npos && currentSection != NULL) {
                // A Key value pair
                string key = line.substr(0, posEquals);
                // TODO trim(key);
                string value = line.substr(posEquals+1, string::npos);
                // TODO trim(value);

                currentSection->insert(pair<string,string>(key,value));
//                (*currentSection)[key] = value; // what the best ?
            }
        }
        currentSection = NULL;
        fileStream.close();
    } else {
        throw FD_EXCEPTION("open " + fileName + " failed.");
    }
}
