#include<dirent.h>
#include<exception>
#include<locale>
#include<iostream>
#include<fstream>
#include "libfreedesktop.h"

using namespace std;

namespace freedesktop {

#ifdef __unix__
  //        const string PATH_SEPARATOR_CHAR = "/";
  //        const string DOUBLE_PATH_SEPARATOR_CHAR = "//";
#elif WIN32
  //        string PATH_SEPARATOR_CHAR = "\\";
  //        string DOUBLE_PATH_SEPARATOR_CHAR = "\\\\";
#endif

  void stringSplit(const string &str, stringList &result, const string &delimiters) {
    string::size_type pos = 0;
    string::size_type lastPos = str.find(delimiters, 0);

    while (lastPos != string::npos) {
      if (pos != lastPos) {
	result.push_back(str.substr(pos, lastPos-pos));
      }
      pos = lastPos+1;
      lastPos = str.find(delimiters, pos);
    }

    if (pos < str.length()) {
      result.push_back(str.substr(pos, str.length()-pos));
    }

  }

  void stringReplace(string &str, const string &oldValue, const string &newValue) {
    string::size_type found = str.find(oldValue);
    while (found != string::npos) {
      str.replace(found, oldValue.length(), newValue);
      found = str.find(oldValue);
    }
  }

  bool stringListContains(const stringList &list, const string &value) {
    bool ret = false;
    for (stringList::size_type i = 0; i < list.size(); i++) {
      if (value == list.at(i)) {
	ret = true;
	i = list.size();
      }
    }
    return ret;
  }

  void printList(const stringList &list) {
#ifdef DEBUG
    cout << endl;
    cout << "\tDEBUG list size: " << list.size() << endl;
    for (stringList::size_type i=0; i<list.size(); i++) {
      cout << "\tDEBUG:" << i << ":" << list.at(i) << endl;
    }
    cout << endl;
#endif
  }

  bool fileExists(const std::string filename) {
    if (!filename.empty()) {
      ifstream fileStream(filename.c_str());
      if (fileStream.is_open()) {
	fileStream.close();
	return true;
      }
    }
    return false;
  }

  bool directoryExists(const std::string dirname) {
    DIR *dir = NULL;

    if (!dirname.empty()) {
      dir = opendir(dirname.c_str());
      if (NULL != dir) {
	(void) closedir(dir);
	dir = NULL;
	return true;
      }
    }
    return false;

  }

  const static locale loc("");

  std::string &stringToLower(std::string &str) {
    if (!str.empty()) {
      for (string::size_type i=0; i<str.size(); i++) {
	str[i] = tolower(str[i], loc);
      }
    }
    return str;
  }

  static string locStr;

  void initLocale() {
    string fullLoc = loc.name();
    if (fullLoc.length() == 2) {
      locStr = tolower(fullLoc, loc) + "_" + toupper(fullLoc, loc);
    } else if (fullLoc.length() >= 5 && fullLoc != "POSIX") {
      locStr = fullLoc.substr(0, 5);
    } else {
      // Default language is most of the time english, and most of the software comes from the US, so ...
      // Used for "C" and "POSIX" locale I think.
      locStr = "en_US";
    }
  }

  int locStringMatchCurrent(const string &locTest) {

    // first, test if the locales variables has been set
    if (locStr.length() == 0) {
      initLocale();
    }

    int ret = 0;
    try {
      ret =  locStr.compare(0, 2, locTest, 0, 2) == 0 ? 1 : 0;
      ret += locStr.compare(3, 2, locTest, 3, 2) == 0 ? 1 : 0;
    } catch (...) {

    }

    return ret;
  }

  const string &getLocale() {
    // first, test if the locales variables has been set
    if (locStr.length() == 0) {
      initLocale();
    }

    return locStr;
  }

  const string getLocaleLang() {
    // first, test if the locales variables has been set
    if (locStr.length() == 0) {
      initLocale();
    }

    return locStr.substr(0, 2);
  }

  const string getLocaleCountry() {
    // first, test if the locales variables has been set
    if (locStr.length() == 0) {
      initLocale();
    }

    return locStr.substr(3, 2);
  }
}
