#ifndef LIBFREEDESKTOP_H
#define LIBFREEDESKTOP_H

#include<vector>
#include<string>

namespace freedesktop {

#ifndef EMPTY_STRING
#define EMPTY_STRING ""
#endif

#ifdef __unix__
    static const std::string DIR_SEPARATOR_CHAR = "/";
    static const std::string DOUBLE_DIR_SEPARATOR_CHAR = "//";
#elif WIN32
    static const std::string DIR_SEPARATOR_CHAR = "\\";
    static const std::string DOUBLE_DIR_SEPARATOR_CHAR = "\\\\";
#endif

    typedef std::vector<std::string> stringList;

    // TODO replace with boost c++ lib ?

    /*!
      \brief A small helper to split a string (sadly the std::string class does not come with a split function.
      */
    void stringSplit(const std::string &str, stringList &result, const std::string &delimiters);

    /*!
      \brief Replace parts of the str string (oldValue), with the newValue content.
     */
    void stringReplace(std::string &str, const std::string &oldValue, const std::string &newValue);

    /**
      \brief Search for a string in a stringList.
      */
    bool stringListContains(const stringList &list, const std::string &value);

    /**
      \brief For debug
      */
    void printList(const stringList &list);

    /**
      \brief Test if the file exists and is readable.
      \param filename The full name of the file to test.
      \return true if exists and readable.
      */
    bool fileExists(const std::string filename);

    /**
      \brief Test if the directory exists and is readable.
      \param dirname The full name of the directory to test
      \return true if exists and readable.
      */
    bool directoryExists(const std::string dirname);

    /**
      \brief Put this string in lowercase
      \return a reference on the param string
      */
    std::string &stringToLower(std::string &str);

    /*!
      Test if the locale string in parameter can match the current locale.
      \return an integer which have this signification:
        \li 0: no match at all (ex: test "de" against "fr_FR")
        \li 1: partial match (ex: test "es_ES" against "es" or "it" against "it_IT")
        \li 2: exact match (ex: "ie_IE" against "ie_IE")
      */
    int locStringMatchCurrent(const std::string &);

    /*!
      \return Return the current locale (always with 5 char: "fr_FR").
      */
    const std::string &getLocale();

    /*!
      \return Return the lang of the current locale: "fr".
      */
    const std::string getLocaleLang();

    /*!
      \return Return the country of the current locale: "FR".
      */
    const std::string getLocaleCountry();
}

#endif // LIBFREEDESKTOP_H
