/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include<iostream>
#include<exception>
#include<stdexcept>
#include "icontheme.h"
#include "iconnaming.h"

using namespace std;
using namespace freedesktop;

bool iconFileFormatsTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    int out = IconTheme::getIconsFilesFormats();
    if ((out & (ICON_FILE_FORMAT_PNG | ICON_FILE_FORMAT_XPM)) != out) {
        cout << ": ko line " << __LINE__ << endl;
        return false;
    }

    IconTheme::setIconsFilesFormats(ICON_FILE_FORMAT_PNG);

    out = IconTheme::getIconsFilesFormats();
    if ((out & ICON_FILE_FORMAT_PNG) != out) {
        cout << ": ko line " << __LINE__ << endl;
        return false;
    }

    // Test bad format, an exception must be throw
    try {
        IconTheme::setIconsFilesFormats(589510);
        cout << ": ko line " << __LINE__ << endl;
    } catch(FDException e) {

    }

    cout << ": ok" << endl;
    return true;
}

bool findThemeTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    // search for the default theme:
    IconTheme *theme = IconTheme::findTheme("hicolor");
    if (NULL == theme) {
        cout << ": ko line " << __LINE__ << endl;
        return false;
    }
    theme->loadTheme();


    cout << ": ok (" << theme->getName() << ")" << endl;
    return true;
}

bool getCurrentThemeTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    IconTheme *theme = IconTheme::getCurrentTheme();
    if (NULL == theme) {
        cout << ": ko line " << __LINE__ << endl;
        return false;
    }
    theme->loadTheme();

    cout << ": ok (" << theme->getName() << ")" << endl;
    return true;
}

bool findIconTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;
    string filename;

//    freedesktop::IconTheme::setIconsFilesFormats(ICON_FILE_FORMAT_PNG | ICON_FILE_FORMAT_SVG | ICON_FILE_FORMAT_XPM);
//    freedesktop::IconTheme::setCurrentTheme(freedesktop::IconTheme::findTheme("Humanity"));

    try {
        filename = IconTheme::findIcon(IconNamingActions::DOCUMENT_NEW, 48);
//        filename = IconTheme::findIcon("firefox", 48);
        if (filename.empty()) {
            cout << ": ko line " << __LINE__ << endl;
            return false;
        }
    } catch (FDException e) {
        cout << ": ko line " << __LINE__ << endl;
        e.printBackTrace();
        return false;
    } catch (...) {
        cout << ": ko line " << __LINE__ << endl;
        return false;
    }

    cout << ": ok (" << filename << ")" << endl;
    return true;
}

int main() {
    int c=0;
    int ko=0;

    c++;
    if (!iconFileFormatsTest()) {
        ko++;
    }

    c++;
    if (!findThemeTest()) {
        ko++;
    }

    c++;
    if (!getCurrentThemeTest()) {
        ko++;
    }

    c++;
    if (!findIconTest()) {
        ko++;
    }

    if (0 == ko) {
        cout << "All " << c << " tests ok" << endl;
    } else {
        cout << "Failed: " << ko << " Success: " << c-ko << endl;
    }

    return 0;
}
