/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<iostream>
#include<libfreedesktop.h>
#include<locale>

using namespace std;
using namespace freedesktop;

bool stringSplitTest() {
  stringList aList;
  string stringToSplit = "a;b;c";
  string stringToSplit2 = ";abc;def;;;;";

  cout << __func__ << ": ";

  // TEST 1
  stringSplit(stringToSplit, aList, ";");
  if (aList.size() != 3) {
    cout << "ko" << endl << __func__ << ":" << __LINE__ << ": expected 3 got " << aList.size() << endl;
    return false;
  }
  if (!(aList[0] == "a" && aList[1] == "b" && aList[2] == "c")) {
    cout << "ko" << endl << __func__ << ":" << __LINE__ << ": list value is not a;b;c" << endl;
    cout << "\ta=" << aList[0] << "\tb=" << aList[1] << "\tc=" << aList[2] << endl;
    return false;
  }

  // TEST 2
  aList.clear();
  stringSplit(stringToSplit2, aList, ";");
  if (aList.size() != 2) {
    cout << "ko" << endl << __func__ << ":" << __LINE__ << ": expected 2 got " << aList.size() << endl;
    return false;
  }
  if (!(aList[0] == "abc" && aList[1] == "def")) {
    cout << "ko" << endl << __func__ << ":" << __LINE__ << endl;
  }

  cout << "ok" << endl;
  return true;
}

bool stringReplaceTest() {
    string strOrigin = "$HOME/.config";
    string strReplaceFrom = "$HOME";
    string strReplaceTo = "/home/user";
    string strExpectedResult = "/home/user/.config";

    cout << __func__ << ": ";

    stringReplace(strOrigin, strReplaceFrom, strReplaceTo);
    if (strOrigin != strExpectedResult) {
        cout << "ko line:" << __LINE__ << ": \"" << strOrigin << "\" != \"" << strExpectedResult << "\"" << endl;
        return false;
    }
    cout << "ok" << endl;
    return true;
}

bool testLocale() {
  //    const std::locale locLoc("");
    const string locActual = getLocale();

    cout << __func__ << ": " << std::flush;

    if (locActual.length() != 5) {
        cout << "ko line:" << __LINE__ << locActual << endl;
        return false;
    }
    string locLang = getLocaleLang();
    if (locLang.length() != 2 || locLang != locActual.substr(0,2)) {
        cout << "ko line:" << __LINE__ << ": " << locLang << endl;
    }

    cout << "ok" << endl;
    return true;
}

int main() {

  bool ret = stringSplitTest();
  if (!ret) {
    return 1;
  }

  ret = stringReplaceTest();
  if (!ret) {
    return 2;
  }

  ret = testLocale();
  if (!ret) {
    return 3;
  }

  return 0;
}
