/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include<string>
#include<stdlib.h>
#include<iostream>
#include<xdgbasedirectory.h>

using namespace std;
using namespace freedesktop;

bool GetDataDirsTest() {
  cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;
  
#ifdef __unix__
    setenv(XDGBaseDirectory::GetEnvXDGDataDirs().c_str(), "/usr/share/:/usr/local/share:/opt/share", 1);
#else
    putenv("XDG_DATA_DIRS=%ProgramFiles%\\freedesktop;%CommonProgramFiles%\\freedesktop;%SystemRoot%\\system32\\freedesktop");
#endif

    stringList dataDirs = XDGBaseDirectory::GetDataDirs();

    if (dataDirs.size() != 3) {
        cout << ": ko" << endl;
        return false;
    }

    cout << ": ok (" << dataDirs.size() << ")" << endl;

    for (size_t i=0; i < dataDirs.size(); i++) {
        cout << "\t" << i << ": " << dataDirs.at(i) << endl;
    }

    return true;
}

bool GetConfigDirsTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;
    stringList configDirs = XDGBaseDirectory::GetConfigDirs();
    if (configDirs.empty()) {
        cout << ": ko" << endl;
        return false;
    }
    cout << ": ok (" << configDirs.size() << ")" << endl;

    for (size_t i=0; i < configDirs.size(); i++) {
        cout << "\t" << i << ": " << configDirs.at(i) << endl;
    }
    return true;
}

bool GetDataHomeTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    string data = XDGBaseDirectory::GetDataHome();
    if (data.empty()) {
        cout << ": ko" << endl;
        return false;
    }

    cout << ": ok: " << data << endl;
    return true;
}

bool GetConfigHomeTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    string config = XDGBaseDirectory::GetConfigHome();
    if (config.empty()) {
        cout << ": ko" << endl;
        return false;
    }

    cout << ": ok: " << config << endl;
    return true;
}

bool GetCacheHomeTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    string cache = XDGBaseDirectory::GetCacheHome();
    if (cache.empty()) {
        cout << ": ko" << endl;
        return false;
    }

    cout << ": ok: " << cache << endl;
    return true;
}

bool GetHomeDirTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    string home = XDGBaseDirectory::GetHomeDir();
    if (home.empty()) {
        cout << ": ko" << endl;
        return false;
    }

    cout << ": ok: " << home << endl;
    return true;
}

bool GetRuntimeDirTest() {

    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;

    string runtimeDir;
    try {
        runtimeDir = XDGBaseDirectory::GetRuntimeDir();

        if (runtimeDir.empty()) {
            cout << ": ko" << endl;
            return false;
        }
    } catch (FDException e) {
        cout << ": ko " << e.what() << endl;
        return false;
    }

    cout << ": ok: " << runtimeDir << endl;
    return true;
}

bool GetXDGDirsTest() {
    cout << __FILE__ << ":" << __LINE__ << ":" << __func__ << flush;
    string xdgDirList = XDGBaseDirectory::GetEnvXDGCacheHome() + ":" + XDGBaseDirectory::GetEnvXDGConfigDirs()
                         + ":" + XDGBaseDirectory::GetEnvXDGConfigHome() + ":" + XDGBaseDirectory::GetEnvXDGDataDirs()
                          + ":" + XDGBaseDirectory::GetEnvXDGDataHome() + ":" + XDGBaseDirectory::GetEnvXDGRuntimeDir();
    // XDGBaseDirectory::
    stringList xdgDirs;
    try {
        xdgDirs = XDGBaseDirectory::GetXDGDirs(xdgDirList, "tests");
    }  catch (FDException e) {
        cout << ": ko" << endl;
        cout << "error: " << e.what() << endl;
        //e.printBackTrace();
        return false;
    }

    if (xdgDirs.empty()) {
        cout << ": ko" << endl;
        return false;
    }
    cout << ": ok (" << xdgDirs.size() << ")" << endl;

    for (size_t i=0; i < xdgDirs.size(); i++) {
        cout << "\t" << i << ": " << xdgDirs.at(i) << endl;
    }
    return true;

}

int main() {
  int c=0;
  int ko = 0;

  c++;
  if (!GetDataDirsTest()) {
    ko++;
  }

  c++;
  if (!GetConfigDirsTest()) {
    ko++;
  }

  c++;
  if (!GetDataHomeTest()) {
      ko++;
  }

  c++;
  if (!GetConfigHomeTest()) {
      ko++;
  }

  c++;
  if (!GetCacheHomeTest()) {
      ko++;
  }

  c++;
  if (!GetHomeDirTest()) {
      ko++;
  }

  c++;
  if (!GetRuntimeDirTest()) {
      ko++;
  }

  c++;
  if (!GetXDGDirsTest()) {
      ko++;
  }

  if (0 == ko) {
      cout << "All " << c << " tests ok" << endl;
  } else {
      cout << "Failed: " << ko << " Success: " << c-ko << endl;
  }

  return 0;
}



