/*
    Copyright (C) 2010 Yvon Tanguy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include<iostream>
#include<fstream>

#include "libfreedesktop.h"
#include "xdguserdirectory.h"

using namespace std;
using namespace freedesktop;

const string XDGUserDirectory::systemFileDirsDefaults = "user-dirs.defaults";
const string XDGUserDirectory::userFileDirsLocale = "user-dirs.locale";
const string XDGUserDirectory::userFileDirsDirs = "user-dirs.dirs";

bool XDGUserDirectory::isXDGDirLoaded = false;

static const string VAR_XDG_DESKTOP_DIR="XDG_DESKTOP_DIR";
static const string VAR_XDG_DOCUMENTS_DIR="XDG_DOCUMENTS_DIR";
static const string VAR_XDG_DOWNLOAD_DIR="XDG_DOWNLOAD_DIR";
static const string VAR_XDG_MUSIC_DIR="XDG_MUSIC_DIR";
static const string VAR_XDG_PICTURES_DIR="XDG_PICTURES_DIR";
static const string VAR_XDG_PUBLICSHARE_DIR="XDG_PUBLICSHARE_DIR";
static const string VAR_XDG_TEMPLATES_DIR="XDG_TEMPLATES_DIR";
static const string VAR_XDG_VIDEOS_DIR="XDG_VIDEOS_DIR";

XDGUserDirectory::XDGUserDirectory() {
}


string XDGUserDirectory::getDirectory(UserDirectories directory) {
    loadUserDirs();

    switch (directory) {
    case XDG_DESKTOP_DIR:
        return xdgDesktopDir;
    case XDG_DOCUMENTS_DIR:
        return xdgDocumentsDir;
    case XDG_DOWNLOAD_DIR:
        return xdgDownloadDir;
    case XDG_MUSIC_DIR:
        return xdgMusicDir;
    case XDG_PICTURES_DIR:
        return xdgPicturesDir;
    case XDG_PUBLICSHARE_DIR:
        return xdgPublicshareDir;
    case XDG_TEMPLATES_DIR:
        return xdgTemplatesDir;
    case XDG_VIDEOS_DIR:
        return xdgVideosDir;
    }

    // Default
    return XDGBaseDirectory::GetHomeDir();
}

string XDGUserDirectory::getDirectoriesLocale() {
    // TODO
    return getLocale();
}

void XDGUserDirectory::loadUserDirs() {

    // TODO thread safe ?
    if (isXDGDirLoaded) {
        // TODO check if the file has change?
        return;
    }

    string xdg_user_config_dir = XDGBaseDirectory::GetConfigHome();
    string user_file_dirs = xdg_user_config_dir + "/" + userFileDirsDirs;
    stringReplace(user_file_dirs, DOUBLE_DIR_SEPARATOR_CHAR, DIR_SEPARATOR_CHAR);

    string line;
    ifstream fileStream(user_file_dirs.c_str(), ios::in);
    if (fileStream.is_open()) {
        while (fileStream.good()) {
            getline(fileStream, line);

            if (line.compare(0, VAR_XDG_DESKTOP_DIR.size(), VAR_XDG_DESKTOP_DIR) == 0) {
                // get the value ( -> "$HOME/Desktop"
                xdgDesktopDir = line.substr(VAR_XDG_DESKTOP_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgDesktopDir, "\"", "");
                xdgDesktopDir = XDGBaseDirectory::EnvPathToRealPath(xdgDesktopDir);
            } else if (line.compare(0, VAR_XDG_DOCUMENTS_DIR.size(), VAR_XDG_DOCUMENTS_DIR) == 0) {
                // get the value ( -> "$HOME/Documents"
                xdgDocumentsDir= line.substr(VAR_XDG_DOCUMENTS_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgDocumentsDir, "\"", "");
                xdgDocumentsDir = XDGBaseDirectory::EnvPathToRealPath(xdgDocumentsDir);
            } else if (line.compare(0, VAR_XDG_DOWNLOAD_DIR.size(), VAR_XDG_DOWNLOAD_DIR) == 0) {
                // get the value ( -> "$HOME/Download"
                xdgDownloadDir = line.substr(VAR_XDG_DOWNLOAD_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgDownloadDir, "\"", "");
                xdgDownloadDir = XDGBaseDirectory::EnvPathToRealPath(xdgDownloadDir);
            } else if (line.compare(0, VAR_XDG_MUSIC_DIR.size(), VAR_XDG_MUSIC_DIR) == 0) {
                // get the value ( -> "$HOME/Music"
                xdgMusicDir = line.substr(VAR_XDG_MUSIC_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgMusicDir, "\"", "");
                xdgMusicDir = XDGBaseDirectory::EnvPathToRealPath(xdgMusicDir);
            } else if (line.compare(0, VAR_XDG_PICTURES_DIR.size(), VAR_XDG_PICTURES_DIR) == 0) {
                // get the value ( -> "$HOME/Pictures"
                xdgPicturesDir = line.substr(VAR_XDG_PICTURES_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgPicturesDir, "\"", "");
                xdgPicturesDir = XDGBaseDirectory::EnvPathToRealPath(xdgPicturesDir);
            } else if (line.compare(0, VAR_XDG_PUBLICSHARE_DIR.size(), VAR_XDG_PUBLICSHARE_DIR) == 0) {
                // get the value ( -> "$HOME/Public"
                xdgPublicshareDir = line.substr(VAR_XDG_PUBLICSHARE_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgPublicshareDir, "\"", "");
                xdgPublicshareDir = XDGBaseDirectory::EnvPathToRealPath(xdgPublicshareDir);
            } else if (line.compare(0, VAR_XDG_TEMPLATES_DIR.size(), VAR_XDG_TEMPLATES_DIR) == 0) {
                // get the value ( -> "$HOME/Templates"
                xdgTemplatesDir = line.substr(VAR_XDG_TEMPLATES_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgTemplatesDir, "\"", "");
                xdgTemplatesDir = XDGBaseDirectory::EnvPathToRealPath(xdgTemplatesDir);
            } else if (line.compare(0, VAR_XDG_VIDEOS_DIR.size(), VAR_XDG_VIDEOS_DIR) == 0) {
                // get the value ( -> "$HOME/Videos"
                xdgVideosDir = line.substr(VAR_XDG_VIDEOS_DIR.size()+1);
                // Remove double quotes
                stringReplace(xdgVideosDir, "\"", "");
                xdgVideosDir = XDGBaseDirectory::EnvPathToRealPath(xdgVideosDir);
            }
            // else ignore
        }
        fileStream.close();
        isXDGDirLoaded = true;
    } else {
        //throw fileStream.exceptions()
    }

}

void XDGUserDirectory::saveUserDirs() {

}
